==> valtio-0.8.0.tgz: package/utils.module.js <==
import { useRef } from 'react';
import { proxy, useProxy, subscribe, snapshot } from 'valtio';
import { createDeepProxy, isDeepChanged } from 'proxy-compare';

/**
 * @deprecated useLocalProxy
 *
 * This is to create a proxy in a component at mount.
 * and discard it when the component unmounts.
 * It returns a tuple of snapshot and state.
 *
 * [Notes]
 * Valtio is designed for module state and this use case for component states
 * is not a primary target. It might not be ideal for such use cases.
 * For component state, alternatively consider using
 * [useImmer](https://github.com/immerjs/use-immer).
 */

const useLocalProxy = init => {
  const ref = useRef();

  if (!ref.current) {
    const initialObject = typeof init === 'function' ? init() : init;
    ref.current = proxy(initialObject);
  }

  return [useProxy(ref.current), ref.current];
};
/**
 * subscribeKey
 *
 * The subscribeKey utility enables subscription to a primitive subproperty of a given state proxy.
 * Subscriptions created with subscribeKey will only fire when the specified property changes.
 *
 * @example
 * import { subscribeKey } from 'valtio/utils'
 * subscribeKey(state, 'count', (v) => console.log('state.count has changed to', v))
 */

const subscribeKey = (proxyObject, key, callback) => {
  let prevValue = proxyObject[key];
  return subscribe(proxyObject, () => {
    const nextValue = proxyObject[key];

    if (!Object.is(prevValue, nextValue)) {
      callback(prevValue = nextValue);
    }
  });
};
/**
 * devtools
 *
 * This is to connect with [Redux DevTools Extension](https://github.com/zalmoxisus/redux-devtools-extension).
 * Limitation: Only plain objects/values are supported.
 *
 * @example
 * import { devtools } from 'valtio/utils'
 * const state = proxy({ count: 0, text: 'hello' })
 * const unsub = devtools(state, 'state name')
 */

const devtools = (proxyObject, name) => {
  let extension;

  try {
    extension = window.__REDUX_DEVTOOLS_EXTENSION__;
  } catch (_unused) {}

  if (!extension) {
    if (typeof process === 'object' && process.env.NODE_ENV === 'development' && typeof window !== 'undefined') {
      console.warn('[Warning] Please install/enable Redux devtools extension');
    }

    return;
  }

  let isTimeTraveling = false;
  const devtools = extension.connect({
    name
  });
  const unsub1 = subscribe(proxyObject, () => {
    if (isTimeTraveling) {
      isTimeTraveling = false;
    } else {
      devtools.send(`Update - ${new Date().toLocaleString()}`, snapshot(proxyObject));
    }
  });
  const unsub2 = devtools.subscribe(message => {
    var _message$payload3;

    if (message.type === 'DISPATCH' && message.state) {
      var _message$payload, _message$payload2;

      if (((_message$payload = message.payload) == null ? void 0 : _message$payload.type) === 'JUMP_TO_ACTION' || ((_message$payload2 = message.payload) == null ? void 0 : _message$payload2.type) === 'JUMP_TO_STATE') {
        isTimeTraveling = true;
      }

      const nextValue = JSON.parse(message.state);
      Object.keys(nextValue).forEach(key => {
        proxyObject[key] = nextValue[key];
      });
    } else if (message.type === 'DISPATCH' && ((_message$payload3 = message.payload) == null ? void 0 : _message$payload3.type) === 'COMMIT') {
      devtools.init(snapshot(proxyObject));
    }
  });
  devtools.init(snapshot(proxyObject));
  return () => {
    unsub1();
    unsub2();
  };
};
/**
 * proxyWithComputed
 *
 * This is to create a proxy with initial object and additional object,
 * which specifies getters for computed values with dependency tracking.
 * It also accepts optional setters for computed values.
 *
 * [Notes]
 * This is for expert users and not recommended for ordinary users.
 * Contradictory to its name, this is costly and overlaps with useSnapshot.
 * Do not try to optimize too early. It can worsen the performance.
 * Measurement and comparison will be very important.
 *
 * @example
 * import { proxyWithComputed } from 'valtio/utils'
 * const state = proxyWithComputed({
 *   count: 1,
 * }, {
 *   doubled: snap => snap.count * 2, // getter only
 *   tripled: {
 *     get: snap => snap.count * 3,
 *     set: (state, newValue) => { state.count = newValue / 3 }
 *   }, // with optional setter
 * })
 */

const proxyWithComputed = (initialObject, computedFns) => {
  const NOTIFIER = Symbol();
  Object.defineProperty(initialObject, NOTIFIER, {
    value: 0
  });
  Object.keys(computedFns).forEach(key => {
    if (Object.getOwnPropertyDescriptor(initialObject, key)) {
      throw new Error('object property already defined');
    }

    const computedFn = computedFns[key];
    const {
      get,
      set
    } = typeof computedFn === 'function' ? {
      get: computedFn
    } : computedFn;
    let computedValue;
    let prevSnapshot;
    let affected = new WeakMap();
    const desc = {};

    desc.get = () => {
      const nextSnapshot = snapshot(proxyObject);

      if (!prevSnapshot || isDeepChanged(prevSnapshot, nextSnapshot, affected)) {
        affected = new WeakMap();
        computedValue = get(createDeepProxy(nextSnapshot, affected));

        if (computedValue instanceof Promise) {
          computedValue.then(v => {
            computedValue = v;
            ++proxyObject[NOTIFIER]; // HACK notify update
          }); // XXX no error handling
        }

        prevSnapshot = nextSnapshot;
      }

      return computedValue;
    };

    if (set) {
      desc.set = newValue => set(proxyObject, newValue);
    }

    Object.defineProperty(initialObject, key, desc);
  });
  const proxyObject = proxy(initialObject);
  return proxyObject;
};

export { devtools, proxyWithComputed, subscribeKey, useLocalProxy };

==> valtio-0.8.1.tgz: package/utils.module.js <==
import { useRef } from 'react';
import { proxy, subscribe, snapshot } from 'valtio/vanilla';
import { useProxy } from 'valtio';
import { createDeepProxy, isDeepChanged } from 'proxy-compare';

/**
 * @deprecated useLocalProxy
 *
 * This is to create a proxy in a component at mount.
 * and discard it when the component unmounts.
 * It returns a tuple of snapshot and state.
 *
 * [Notes]
 * Valtio is designed for module state and this use case for component states
 * is not a primary target. It might not be ideal for such use cases.
 * For component state, alternatively consider using
 * [useImmer](https://github.com/immerjs/use-immer).
 */

const useLocalProxy = init => {
  const ref = useRef();

  if (!ref.current) {
    const initialObject = typeof init === 'function' ? init() : init;
    ref.current = proxy(initialObject);
  }

  return [useProxy(ref.current), ref.current];
};
/**
 * subscribeKey
 *
 * The subscribeKey utility enables subscription to a primitive subproperty of a given state proxy.
 * Subscriptions created with subscribeKey will only fire when the specified property changes.
 * notifyInSync: same as the parameter to subscribe(); true disables batching of subscriptions.
 *
 * @example
 * import { subscribeKey } from 'valtio/utils'
 * subscribeKey(state, 'count', (v) => console.log('state.count has changed to', v))
 */

const subscribeKey = (proxyObject, key, callback, notifyInSync) => {
  let prevValue = proxyObject[key];
  return subscribe(proxyObject, () => {
    const nextValue = proxyObject[key];

    if (!Object.is(prevValue, nextValue)) {
      callback(prevValue = nextValue);
    }
  }, notifyInSync);
};
/**
 * devtools
 *
 * This is to connect with [Redux DevTools Extension](https://github.com/zalmoxisus/redux-devtools-extension).
 * Limitation: Only plain objects/values are supported.
 *
 * @example
 * import { devtools } from 'valtio/utils'
 * const state = proxy({ count: 0, text: 'hello' })
 * const unsub = devtools(state, 'state name')
 */

const devtools = (proxyObject, name) => {
  let extension;

  try {
    extension = window.__REDUX_DEVTOOLS_EXTENSION__;
  } catch (_unused) {}

  if (!extension) {
    if (typeof process === 'object' && process.env.NODE_ENV === 'development' && typeof window !== 'undefined') {
      console.warn('[Warning] Please install/enable Redux devtools extension');
    }

    return;
  }

  let isTimeTraveling = false;
  const devtools = extension.connect({
    name
  });
  const unsub1 = subscribe(proxyObject, () => {
    if (isTimeTraveling) {
      isTimeTraveling = false;
    } else {
      devtools.send(`Update - ${new Date().toLocaleString()}`, snapshot(proxyObject));
    }
  });
  const unsub2 = devtools.subscribe(message => {
    var _message$payload3;

    if (message.type === 'DISPATCH' && message.state) {
      var _message$payload, _message$payload2;

      if (((_message$payload = message.payload) == null ? void 0 : _message$payload.type) === 'JUMP_TO_ACTION' || ((_message$payload2 = message.payload) == null ? void 0 : _message$payload2.type) === 'JUMP_TO_STATE') {
        isTimeTraveling = true;
      }

      const nextValue = JSON.parse(message.state);
      Object.keys(nextValue).forEach(key => {
        proxyObject[key] = nextValue[key];
      });
    } else if (message.type === 'DISPATCH' && ((_message$payload3 = message.payload) == null ? void 0 : _message$payload3.type) === 'COMMIT') {
      devtools.init(snapshot(proxyObject));
    }
  });
  devtools.init(snapshot(proxyObject));
  return () => {
    unsub1();
    unsub2();
  };
};
/**
 * addComputed
 *
 * This adds computed values to an existing proxy object.
 *
 * [Notes]
 * This is for expert users and not recommended for ordinary users.
 * Contradictory to its name, this is costly and overlaps with useSnapshot.
 * Do not try to optimize too early. It can worsen the performance.
 * Measurement and comparison will be very important.
 *
 * @example
 * import { proxy } from 'valtio'
 * import { addComputed } from 'valtio/utils'
 * const state = proxy({
 *   count: 1,
 * })
 * addComputed(state, {
 *   doubled: snap => snap.count * 2,
 * })
 */

const addComputed = (proxyObject, computedFns, targetObject = proxyObject) => {
  Object.keys(computedFns).forEach(key => {
    if (Object.getOwnPropertyDescriptor(targetObject, key)) {
      throw new Error('object property already defined');
    }

    const get = computedFns[key];
    let prevSnapshot;
    let affected = new WeakMap();
    let pending = false;

    const callback = () => {
      const nextSnapshot = snapshot(proxyObject);

      if (!pending && (!prevSnapshot || isDeepChanged(prevSnapshot, nextSnapshot, affected))) {
        affected = new WeakMap();
        const value = get(createDeepProxy(nextSnapshot, affected));
        prevSnapshot = nextSnapshot;

        if (value instanceof Promise) {
          pending = true;
          value.then(v => {
            targetObject[key] = v;
          }).catch(e => {
            // not ideal but best effort for throwing error with proxy
            targetObject[key] = new Proxy({}, {
              get() {
                throw e;
              }

            });
          }).finally(() => {
            pending = false;
          });
        }

        targetObject[key] = value;
      }
    };

    subscribe(proxyObject, callback, true);
    callback();
  });
};
/**
 * proxyWithComputed
 *
 * This is to create a proxy with initial object and additional object,
 * which specifies getters for computed values with dependency tracking.
 * It also accepts optional setters for computed values.
 *
 * [Notes]
 * This is for expert users and not recommended for ordinary users.
 * Contradictory to its name, this is costly and overlaps with useSnapshot.
 * Do not try to optimize too early. It can worsen the performance.
 * Measurement and comparison will be very important.
 *
 * @example
 * import { proxyWithComputed } from 'valtio/utils'
 * const state = proxyWithComputed({
 *   count: 1,
 * }, {
 *   doubled: snap => snap.count * 2, // getter only
 *   tripled: {
 *     get: snap => snap.count * 3,
 *     set: (state, newValue) => { state.count = newValue / 3 }
 *   }, // with optional setter
 * })
 */

const proxyWithComputed = (initialObject, computedFns) => {
  Object.keys(computedFns).forEach(key => {
    if (Object.getOwnPropertyDescriptor(initialObject, key)) {
      throw new Error('object property already defined');
    }

    const computedFn = computedFns[key];
    const {
      get,
      set
    } = typeof computedFn === 'function' ? {
      get: computedFn
    } : computedFn;
    let computedValue;
    let prevSnapshot;
    let affected = new WeakMap();
    const desc = {};

    desc.get = () => {
      const nextSnapshot = snapshot(proxyObject);

      if (!prevSnapshot || isDeepChanged(prevSnapshot, nextSnapshot, affected)) {
        affected = new WeakMap();
        computedValue = get(createDeepProxy(nextSnapshot, affected));
        prevSnapshot = nextSnapshot;
      }

      return computedValue;
    };

    if (set) {
      desc.set = newValue => set(proxyObject, newValue);
    }

    Object.defineProperty(initialObject, key, desc);
  });
  const proxyObject = proxy(initialObject);
  return proxyObject;
};

export { addComputed, devtools, proxyWithComputed, subscribeKey, useLocalProxy };
